indexing

	description: "selection of a single value using a set function"
	author:     "Berend de Boer <berend@pobox.com>"
	copyright:  "Copyright (c) 1999, Berend de Boer"
	date:       "$Date: 2008/07/26 $"
	revision:   "$Revision: #10 $"

class

	XPLAIN_SELECTION_FUNCTION

inherit

	XPLAIN_SELECTION_VALUE

create

	make


feature {NONE} -- Initialization

	make (
			a_function: XPLAIN_FUNCTION
			a_subject: XPLAIN_SUBJECT
			a_property: XPLAIN_EXPRESSION
			a_predicate: XPLAIN_EXPRESSION) is
		require
			function_not_void: a_function /= Void
			subject_not_void: a_subject /= Void
		do
			function := a_function
			subject := a_subject
			property := a_property
			predicate := a_predicate
		end


feature -- Access

	function: XPLAIN_FUNCTION
			-- The particular Xplain function


feature -- SQL generation

	representation (sqlgenerator: SQL_GENERATOR): XPLAIN_REPRESENTATION is
			-- Suitable Xplain representation for this function.
		do
			Result := function.representation (sqlgenerator, type, property)
		end

	sqlvalue (sqlgenerator: SQL_GENERATOR): STRING is
			-- The sql expression that returns a single value
		do
			Result := sqlgenerator.sql_select_function_as_subselect (Current)
		end

	sp_function_type (sqlgenerator: SQL_GENERATOR): STRING is
			-- Callback in generator to generate function type for
			-- PostgreSQL functions.
		do
			Result := function.sp_function_type (sqlgenerator, Current)
		end

	supports_left_outer_join: BOOLEAN is
			-- Does this expression support being written as a left outer
			-- join instead of a subselect? Left outer joins are faster
			-- with most optimizers.
		local
			some: XPLAIN_SOME_FUNCTION
		do
			Result := not function.is_existential -- and then predicate = Void
			if Result then
				-- The some function can use a left outer join if it returns
				-- max 1 row.
				some ?= function
				if some /= Void then
					Result := predicate /= Void and then predicate.selects_max_one_row
				end
			end
		ensure
			cannot_be_existential: Result implies not function.is_existential
		end

	write_select (a_generator: ABSTRACT_GENERATOR) is
		do
			a_generator.write_select_function (Current)
		end


invariant

	have_function: function /= Void

end
